@@ -37,6 +37,7 @@
 #include <ros/duration.h>
 #include <ros/rate.h>
 #include <boost/date_time/posix_time/posix_time_types.hpp>
+#include <boost/math/special_functions/round.hpp>
 
 namespace ros {
   //
@@ -53,21 +54,27 @@ namespace ros {
   T& DurationBase<T>::fromSec(double d)
   {
     int64_t sec64 = (int64_t)floor(d);
-    if (sec64 < INT_MIN || sec64 > INT_MAX)
+    if (sec64 < std::numeric_limits<int32_t>::min() || sec64 > std::numeric_limits<int32_t>::max())
       throw std::runtime_error("Duration is out of dual 32-bit range");
     sec = (int32_t)sec64;
-    nsec = (int32_t)(nearbyint((d - (double)sec)*1000000000));
+    nsec = (int32_t)boost::math::round((d - sec) * 1e9);
+    int32_t rollover = nsec / 1000000000ul;
+    sec += rollover;
+    nsec %= 1000000000ul;
     return *static_cast<T*>(this);
   }
 
   template<class T>
   T& DurationBase<T>::fromNSec(int64_t t)
   {
-    int64_t sec64 = t / 1000000000;
-    if (sec64 < INT_MIN || sec64 > INT_MAX)
+    int64_t sec64 = t / 1000000000LL;
+    if (sec64 < std::numeric_limits<int32_t>::min() || sec64 > std::numeric_limits<int32_t>::max())
       throw std::runtime_error("Duration is out of dual 32-bit range");
     sec = (int32_t)sec64;
-    nsec = (int32_t)(t % 1000000000);
+    nsec = (int32_t)(t % 1000000000LL);
+
+    normalizeSecNSecSigned(sec, nsec);
+
     return *static_cast<T*>(this);
   }
 
