@@ -42,8 +42,8 @@
 #include <cmath>
 #include <ctime>
 #include <iomanip>
-#include <stdexcept>
 #include <limits>
+#include <stdexcept>
 
 // time related includes for macOS
 #if defined(__APPLE__)
@@ -51,6 +51,12 @@
 #include <mach/mach.h>
 #endif  // defined(__APPLE__)
 
+#ifdef _WINDOWS
+#include <chrono>
+#include <thread>
+#include <windows.h>
+#endif
+
 #include <boost/thread/mutex.hpp>
 #include <boost/io/ios_state.hpp>
 #include <boost/date_time/posix_time/ptime.hpp>
@@ -101,14 +107,14 @@ namespace ros
 #if HAS_CLOCK_GETTIME
     timespec start;
     clock_gettime(CLOCK_REALTIME, &start);
-    if (start.tv_sec < 0 || start.tv_sec > UINT_MAX)
+    if (start.tv_sec < 0 || start.tv_sec > std::numeric_limits<uint32_t>::max())
       throw std::runtime_error("Timespec is out of dual 32-bit range");
     sec  = start.tv_sec;
     nsec = start.tv_nsec;
 #else
     struct timeval timeofday;
     gettimeofday(&timeofday,NULL);
-    if (timeofday.tv_sec < 0 || timeofday.tv_sec > UINT_MAX)
+    if (timeofday.tv_sec < 0 || timeofday.tv_sec > std::numeric_limits<uint32_t>::max())
       throw std::runtime_error("Timeofday is out of dual signed 32-bit range");
     sec  = timeofday.tv_sec;
     nsec = timeofday.tv_usec * 1000;
@@ -148,7 +154,7 @@ namespace ros
     	start_li.QuadPart -= 116444736000000000ULL;
 #endif
         int64_t start_sec64 = start_li.QuadPart / 10000000;  // 100-ns units
-        if (start_sec64 < 0 || start_sec64 > UINT_MAX)
+        if (start_sec64 < 0 || start_sec64 > std::numeric_limits<uint32_t>::max())
           throw std::runtime_error("SystemTime is out of dual 32-bit range");
         start_sec = (uint32_t)start_sec64;
         start_nsec = (start_li.LowPart % 10000000) * 100;
@@ -228,27 +234,7 @@ namespace ros
   int ros_nanosleep(const uint32_t &sec, const uint32_t &nsec)
   {
 #if defined(WIN32)
-    HANDLE timer = NULL;
-    LARGE_INTEGER sleepTime;
-    sleepTime.QuadPart = -
-      static_cast<int64_t>(sec)*10000000LL -
-      static_cast<int64_t>(nsec) / 100LL;
-
-    timer = CreateWaitableTimer(NULL, TRUE, NULL);
-    if (timer == NULL)
-      {
-        return -1;
-      }
-
-    if (!SetWaitableTimer (timer, &sleepTime, 0, NULL, NULL, 0))
-      {
-        return -1;
-      }
-
-    if (WaitForSingleObject (timer, INFINITE) != WAIT_OBJECT_0)
-      {
-        return -1;
-      }
+    std::this_thread::sleep_for(std::chrono::nanoseconds(static_cast<int64_t>(sec * 1e9 + nsec)));
     return 0;
 #else
     timespec req = { sec, nsec };
@@ -376,7 +362,7 @@ namespace ros
   {
     Time t;
     int64_t sec64 = d.total_seconds();
-    if (sec64 < 0 || sec64 > UINT_MAX)
+    if (sec64 < 0 || sec64 > std::numeric_limits<uint32_t>::max())
       throw std::runtime_error("time_duration is out of dual 32-bit range");
     t.sec = (uint32_t)sec64;
 #if defined(BOOST_DATE_TIME_HAS_NANOSECONDS)
@@ -552,7 +538,7 @@ namespace ros
     uint64_t nsec_part = nsec % 1000000000UL;
     uint64_t sec_part = nsec / 1000000000UL;
 
-    if (sec + sec_part > UINT_MAX)
+    if (sec + sec_part > std::numeric_limits<uint32_t>::max())
       throw std::runtime_error("Time is out of dual 32-bit range");
 
     sec += sec_part;
@@ -580,7 +566,7 @@ namespace ros
         --sec_part;
       }
 
-    if (sec_part < 0 || sec_part > UINT_MAX)
+    if (sec_part < 0 || sec_part > std::numeric_limits<uint32_t>::max())
       throw std::runtime_error("Time is out of dual 32-bit range");
 
     sec = sec_part;
